@@ -39,6 +39,7 @@ class BoundaryConditions(object):
     def set(self, type, colors, fcts=None):
         if isinstance(colors, int): colors = [colors]
         for i,color in enumerate(colors):
+            if color in self.type.keys(): raise ValueError(f"attempt to defone {type=}, butalready defined b.c {color} as {self.type[color]=}")
             self.type[color] = type
             if fcts: self.fct[color] = fcts[i]
     def colorsOfType(self, type):
@@ -49,7 +50,9 @@ class BoundaryConditions(object):
     def check(self, colors):
         colors = set(colors)
         typecolors = set(self.type.keys())
-        _check2setsequal_(colors, typecolors, "mesh colors", "types")
+        if colors != typecolors: raise ValueError(f"problem in boundary conditions {colors=} {typecolors=}")
+        # _check2setsequal_(colors, typecolors, "mesh colors", "types")
+            
 
 
 # ---------------------------------------------------------------- #
@@ -68,7 +71,7 @@ class PostProcess(object):
         self.type = {}
         self.color = {}
     def set(self, name, type, colors):
-        colors = list(colors)
+        if isinstance(colors, int): colors=[colors]
         self.type[name] = type
         self.color[name] = colors
     def colors(self, name):
@@ -99,10 +102,10 @@ class Params(object):
         self.scal_cells = {}
     def __repr__(self):
         repr = ""
-        if len(self.fct_glob): repr += f"fct_glob={self.fct_glob}"
-        if len(self.scal_glob): repr += f"scal_glob={self.scal_glob}"
-        if len(self.scal_cells): repr += f"scal_cells={self.scal_cells}"
-        return repr
+        if len(self.fct_glob): repr += f" fct_glob={self.fct_glob}"
+        if len(self.scal_glob): repr += f" scal_glob={self.scal_glob}"
+        if len(self.scal_cells): repr += f" scal_cells={self.scal_cells}"
+        return repr[1:]
     def set_scal_cells(self, name, colors, value):
         if not name in self.scal_cells: self.scal_cells[name]={}
         for color in colors: self.scal_cells[name][color] = value
@@ -186,15 +189,39 @@ class Results(object):
     - point_data, side_data, cell_data, gobal_data
     - info on iteration
     """
+    # TODO: data per physical identity
     def __init__(self):
-        self.data = {"point":{}, "side":{}, "cell":{}, "global":{}}
+        # self.data = {"point":{}, "side":{}, "cell":{}, "global":{}}
+        self.data = {}
         self.info = {}
     def __repr__(self):
         return f"{self.data=}\n {self.info=}"
-    def setData(self, data):
-        if len(data) != 4:
-            raise ValueError("expect four data (point, side, cell, global)")
-        self.data["point"] = data[0]
-        self.data["side"] = data[1]
-        self.data["cell"] = data[2]
-        self.data["global"] = data[3]
\ No newline at end of file
+    def setData(self, data, timer=None, iter=None):
+        self.data = data
+        if timer is not None: self.info['timer'] = timer
+        if iter is not None: self.info['iter'] = iter
+        # if len(data) != 4:
+        #     raise ValueError("expect four data (point, side, cell, global)")
+        # self.data["point"] = data[0]
+        # self.data["side"] = data[1]
+        # self.data["cell"] = data[2]
+        # self.data["global"] = data[3]
+    def addData(self, data, time=None, iter=None):
+        if not len(self.data.keys()):
+            for k,v in data.items():
+                # print(f"{k=} {type(v)=}")
+                if isinstance(v,dict):
+                    self.data[k] = {}
+                    for k2,v2 in v.items():
+                        # print(f"{k2=} {type(v2)=}")
+                        if not isinstance(v2, dict):
+                            self.data[k][k2] = []
+                        else:
+                            assert 0
+                else:
+                    assert 0
+        for k,v in data.items():
+            if isinstance(v,dict):
+                for k2,v2 in v.items():
+                    if not isinstance(v2, dict):
+                        self.data[k][k2].append(v2)
