@@ -1,20 +1,39 @@
 import numpy as np
 import matplotlib.pyplot as plt
 
+#=================================================================#
+def plotMeshWithPointData(ax, pdn, pd, x, alpha):
+    if not isinstance(pd, np.ndarray):
+        raise ValueError(f"Problem in data {type(pd)=}")
+    if x.shape != pd.shape:
+        raise ValueError(f"Problem in data {x.shape=} {pd.shape=}")
+    i = np.argsort(x)
+    ax.plot(x, np.zeros_like(x), "x")
+    ax.plot(x[i], pd[i], 'x-', label=pdn)
+    ax.legend()
+    # clb.set_label(pdn)
+#=================================================================#
+def plotMeshWithCellData(ax, cdn, cd, x, alpha):
+    i = np.argsort(x)
+    xi = x[i]
+    xc = 0.5 * (xi[:-1] + xi[1:])
+    if xc.shape != cd.shape:
+        raise ValueError(f"Problem in data {xc.shape=} {cd.shape=}")
+    ax.plot(xc, cd, 'x-', label=cdn)
+    ax.legend()
+
+
 def plotmesh(mesh, **kwargs):
-    ax = plt
-    if 'ax' in kwargs: ax = kwargs.pop('ax')
+    ax = kwargs.pop('ax', plt)
     x, lines = mesh.points[:, 0], mesh.simplices
     ax.plot(x, np.zeros_like(x), "x-")
 
 def meshWithBoundaries(x, lines, **kwargs):
-    ax = plt
-    if 'ax' in kwargs: ax = kwargs.pop('ax')
+    ax = kwargs.pop('ax', plt)
     ax.plot(x, np.zeros_like(x), "x-")
 
 def meshWithData(**kwargs):
-    ax = plt
-    if 'ax' in kwargs: ax = kwargs.pop('ax')
+    ax = kwargs.pop('ax', plt)
     x = kwargs.pop('x')
     title, suptitle = None, None
     if 'title' in kwargs: title = kwargs['title']
