@@ -9,9 +9,6 @@ import meshio
 import numpy as np
 from scipy import sparse
 from simfempy.tools import npext
-from simfempy.tools import timer
-
-from .testmeshes import __pygmsh6__
 
 #=================================================================#
 class SimplexMesh(object):
@@ -34,6 +31,9 @@ class SimplexMesh(object):
              normals on boundary are external
     sigma: orientation of normal per cell and face (ncells, dimension+1)
 
+    innerfaces: mask for interior faces
+    cellsOfInteriorFaces: cellsOfFaces[innerfaces]
+
     dV: shape (ncells), volumes of simplices
     bdrylabels: dictionary(keys: colors, values: id's of boundary faces)
     cellsoflabel: dictionary(keys: colors, values: id's of cells)
@@ -45,7 +45,6 @@ class SimplexMesh(object):
         s += f"\nbdrylabels={list(self.bdrylabels.keys())}"
         s += f"\ncellsoflabel={list(self.cellsoflabel.keys())}"
         return s
-
     def __init__(self, **kwargs):
         if 'mesh' in kwargs:
             self.geometry = 'own'
@@ -53,6 +52,18 @@ class SimplexMesh(object):
         else:
             raise KeyError("Needs a mesh (no longer geometry)")
         self._initMeshPyGmsh(mesh)
+        self.check()
+    def check(self):
+        if len(np.unique(self.simplices)) != self.nnodes:
+            raise ValueError(f"{len(np.unique(self.simplices))=} BUT {self.nnodes=}")
+    def bdryFaces(self, colors=None):
+        if colors is None: colors = self.bdrylabels.keys()
+        pos = [0]
+        for color in colors: pos.append(pos[-1]+len(self.bdrylabels[color]))
+        faces = np.empty(pos[-1], dtype=np.uint32)
+        for i,color in enumerate(colors): faces[pos[i]:pos[i+1]] = self.bdrylabels[color]
+        return faces
+
     def _initMeshPyGmsh(self, mesh):
         # only 3d-coordinates
         assert mesh.points.shape[1] ==3
@@ -78,10 +89,18 @@ class SimplexMesh(object):
             if key == self.simplicesname: self.simplices = cellblock
             if key == 'vertex': self.vertices = cellblock
             if key == self.facesname:
+                self.facesdata = cellblock
                 # print(f"{key=} {cellblock=}")
                 bdryfacesgmshlist.extend(cellblock)
         if not hasattr(self,"simplices"):
             raise ValueError(f"something wrong {self.dimension=}")
+        # eliminate drangling points
+        nnp = len(np.unique(self.simplices))
+        if nnp != self.nnodes:
+            assert np.all(np.unique(self.simplices)==np.arange(nnp))
+            self.points = self.points[:nnp]
+            self.nnodes = nnp
+        # boundaries
         bdryfacesgmsh = np.array(bdryfacesgmshlist)
         self._constructFacesFromSimplices()
         assert self.dimension+1 == self.simplices.shape[1]
@@ -89,11 +108,12 @@ class SimplexMesh(object):
         self.pointsc = self.points[self.simplices].mean(axis=1)
         self.pointsf = self.points[self.faces].mean(axis=1)
         self._constructNormalsAndAreas()
-        if __pygmsh6__:
-            self._initMeshPyGmsh6(mesh.cells, mesh.cell_data['gmsh:physical'], bdryfacesgmsh)
-        else:
-            self._initMeshPyGmsh7(mesh.cells, mesh.cell_sets, bdryfacesgmsh)
-
+        self.cell_sets = mesh.cell_sets
+        self._initMeshPyGmsh7(mesh.cells, mesh.cell_sets, bdryfacesgmsh)
+        #TODO : remplacer -1 par nan dans les indices
+    def constructInnerFaces(self):
+        self.innerfaces = self.cellsOfFaces[:,1]>=0
+        self.cellsOfInteriorFaces= self.cellsOfFaces[self.innerfaces]
     def _initMeshPyGmsh7(self, cells, cell_sets, bdryfacesgmsh):
         # cell_sets: dict label --> list of None or np.array for each cell_type
         # the indices of the np.array are not the cellids !
@@ -104,15 +124,16 @@ class SimplexMesh(object):
         cellsoflabel = {key:{} for key in self.celltypes}
         ctorderd = []
         for label, cb in cell_sets.items():
+            if label=='gmsh:bounding_entities': continue
             # print(f"{label=} {cb=}")
             if len(cb) != len(self.celltypes): raise KeyError(f"mismatch {label=}")
             for celltype, info in zip(self.celltypes, cb):
                 # only one is supposed to be not None
                 if info is not None:
-                    # print(f"{label=} {type(label)=}")
                     try: ilabel=int(label)
                     except: raise ValueError(f"cannot convert to int {label=} {cell_sets=}")
                     cellsoflabel[celltype][ilabel] = info
+                    # print(f"{label=} {type(label)=} {info=}")
                     sizes[celltype] += info.shape[0]
                     typesoflabel[ilabel] = celltype
                     ctorderd.append(celltype)
@@ -124,37 +145,14 @@ class SimplexMesh(object):
             n += sizes[ct]
         self.cellsoflabel = cellsoflabel[self.simplicesname]
         self.verticesoflabel = {}
-        if self.dimension > 1: self.verticesoflabel = cellsoflabel['vertex']
+        # print(f"{cellsoflabel=}\n{cellsoflabel.keys()}")
+        # if self.dimension > 1: self.verticesoflabel = cellsoflabel['vertex']
         # print(f"{self.verticesoflabel=}")
         # bdry faces
         # for key, cellblock in cells:
         #     if key == self.facesnames[self.dimension - 1]: bdryfacesgmsh = cellblock
         bdrylabelsgmsh = cellsoflabel[self.facesname]
         self._constructBoundaryFaces7(bdryfacesgmsh, bdrylabelsgmsh)
-
-    def _initMeshPyGmsh6(self, cells, cdphys, bdryfacesgmsh):
-        if len(cdphys) != len(self.celltypes):
-            raise KeyError(f"not enough physical labels:\n {self.celltypes=}\n {len(cdphys)=}")
-        # first attempt, 'np.append' copies data...
-        _cells = {}
-        _labels = {}
-        for (key, cellblock), cd in zip(cells,cdphys):
-            if len(cellblock) != len(cd):
-                raise ValueError(f"mismatch in {key} {len(cellblock)} {len(cd)}")
-            if not key in _cells.keys():
-                _cells[key] = cellblock
-                _labels[key] = cd
-            else:
-                _cells[key] = np.append(_cells[key], cellblock, axis=0)
-                _labels[key] = np.append(_labels[key], cd, axis=0)
-        self._constructBoundaryFaces6(bdryfacesgmsh, _labels[self.facesname])
-        # self.cellsoflabel = npext.creatdict_unique_all(self.cell_labels)
-        self.cellsoflabel = npext.creatdict_unique_all(_labels[self.simplicesname])
-        self.verticesoflabel = {}
-        if self.dimension > 1 and 'vertex' in _cells.keys():
-            self.vertices = _cells['vertex'].reshape(-1)
-            self.verticesoflabel = npext.creatdict_unique_all(_labels['vertex'])
-
     def _constructFacesFromSimplices(self):
         simplices = self.simplices
         ncells = simplices.shape[0]
@@ -185,6 +183,7 @@ class SimplexMesh(object):
             self.facesOfCells[cell, loc] = f
             if self.cellsOfFaces[f,0] == -1: self.cellsOfFaces[f,0] = cell
             else: self.cellsOfFaces[f,1] = cell
+
     def _constructBoundaryFaces7(self, bdryfacesgmsh, bdrylabelsgmsh):
         # bdries
         # bdryfacesgmsh may contains interior edges for len(celllabels)>1
@@ -211,26 +210,14 @@ class SimplexMesh(object):
             fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
         # print(f"{bp=}")
         # print(f"{fp=}")
-
 #https://stackoverflow.com/questions/51352527/check-for-identical-rows-in-different-numpy-arrays
         indices = (bdryfacesgmsh[bp, None] == bdryfaces[fp]).all(-1).any(-1)
-        # indices = np.isin(bdryfacesgmsh, bdryfaces)
-        # print(f"{indices=}")
-
-        # fp2 = np.searchsorted(bdryfacesgmsh, bdryfaces, sorter=bp)
-        # print(f"{fp2=}")
-
-        # print(f"{bp[indices]=}")
-        # print(f"{bdryfacesgmsh[bp[indices]]=}")
-        # print(f"{bdryfaces[fp]=}")
         if not np.all(bdryfaces[fp]==bdryfacesgmsh[bp[indices]]):
             raise ValueError(f"{bdryfaces.T=}\n{bdryfacesgmsh.T=}\n{indices=}\n{bdryfaces[fp].T=}\n{bdryfacesgmsh[bp[indices]].T=}")
-
         bp2 = bp[indices]
         for i in range(len(fp)):
             if not np.all(bdryfacesgmsh[bp2[i]] == bdryfaces[fp[i]]):
                 raise ValueError(f"{i=} {bdryfacesgmsh[bp2[i]]=} {bdryfaces[fp[i]]=}")
-
         bpi = np.argsort(bp)
         # bp2i = {bp2[i]:i for i in range(len(bp2))}
         # print(f"{bp=} \n{bp2=} \n{bpi=} \n{bp2i=} \n{indices=}")
@@ -249,81 +236,6 @@ class SimplexMesh(object):
                 for i in range(len(cb)): self.bdrylabels[int(col)][i] = bdryids[fp[binv[cb[i]]]]
             # else:
             #     assert not indices[bpi[cb[0]]]
-
-
-
-        # nbdrylabelsgmsh = 0
-        # for col, cb in bdrylabelsgmsh.items(): nbdrylabelsgmsh += cb.shape[0]
-        # print(f"{nbdrylabelsgmsh=} {nbdryfaces=}")
-        # # if nbdrylabelsgmsh != nbdryfaces:
-        # #     raise ValueError(f"wrong number of boundary labels {nbdrylabelsgmsh=} != {nbdryfaces=}")
-        # if len(bdryfacesgmsh) != nbdryfaces:
-        #     print("wrong number of bdryfaces {} != {}".format(len(bdryfacesgmsh), nbdryfaces))
-        #     # raise ValueError("wrong number of bdryfaces {} != {}".format(len(bdryfacesgmsh), nbdryfaces))
-        # self.bdrylabels = {}
-        # # colorofbdr = -np.ones(nbdryfaces, dtype=bdryids.dtype)
-        # colorofbdr = -np.ones(nbdrylabelsgmsh, dtype=bdryids.dtype)
-        # for col, cb in bdrylabelsgmsh.items():
-        #     self.bdrylabels[int(col)] = -np.ones( (cb.shape[0]), dtype=np.int32)
-        #     colorofbdr[cb] = col
-        #
-        # # perm = bdryids[bp[fpi]]
-        # # print(f"{perm=}")
-        # # print(f"{colorofbdr=}")
-        # counts = {}
-        # for key in list(self.bdrylabels.keys()): counts[key]=0
-        # for i in range(len(perm)):
-        #     if np.any(bdryfacesgmsh[i] != self.faces[perm[i]]):
-        #         raise ValueError(f"Did not find boundary indices\n{bdryfacesgmsh[i]} {self.faces[perm[i]]}")
-        #     color = colorofbdr[i]
-        #     self.bdrylabels[color][counts[color]] = perm[i]
-        #     counts[color] += 1
-        # # print ("self.bdrylabels", self.bdrylabels)
-        # for col, bl in self.bdrylabels.items():
-        #     print(f"{bl=} {self.bdrylabels2[col]=}")
-        #     # assert np.all(bl == self.bdrylabels2[col])
-
-
-    def _constructBoundaryFaces6(self, bdryfacesgmsh, bdrylabelsgmsh):
-        # bdries
-        bdryids = np.flatnonzero(self.cellsOfFaces[:,1] == -1)
-        assert np.all(bdryids == np.flatnonzero(np.any(self.cellsOfFaces == -1, axis=1)))
-        bdryfaces = np.sort(self.faces[bdryids],axis=1)
-        nbdryfaces = len(bdryids)
-        if len(bdrylabelsgmsh) != nbdryfaces:
-            raise ValueError("wrong number of boundary labels {} != {}".format(len(bdrylabelsgmsh),nbdryfaces))
-        if len(bdryfacesgmsh) != nbdryfaces:
-            raise ValueError("wrong number of bdryfaces {} != {}".format(len(bdryfacesgmsh), nbdryfaces))
-        self.bdrylabels = {}
-        colors, counts = np.unique(bdrylabelsgmsh, return_counts=True)
-        # print ("colors, counts", colors, counts)
-        for i in range(len(colors)):
-            self.bdrylabels[colors[i]] = -np.ones( (counts[i]), dtype=np.int32)
-        bdryfacesgmsh = np.sort(bdryfacesgmsh)
-        nnpc = self.simplices.shape[1]
-        s = "{0}" + (nnpc-2)*", {0}"
-        dtb = s.format(bdryfacesgmsh.dtype)
-        dtf = s.format(bdryfaces.dtype)
-        order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
-        if self.dimension==1:
-            bp = np.argsort(bdryfacesgmsh.view(dtb), axis=0).ravel()
-            fp = np.argsort(bdryfaces.view(dtf), axis=0).ravel()
-        else:
-            bp = np.argsort(bdryfacesgmsh.view(dtb), order=order, axis=0).ravel()
-            fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
-        bpi = np.empty(bp.size, bp.dtype)
-        bpi[bp] = np.arange(bp.size)
-        perm = bdryids[fp[bpi]]
-        counts = {}
-        for key in list(self.bdrylabels.keys()): counts[key]=0
-        for i in range(len(perm)):
-            if np.any(bdryfacesgmsh[i] != self.faces[perm[i]]):
-                raise ValueError("Did not find boundary indices")
-            color = bdrylabelsgmsh[i]
-            self.bdrylabels[color][counts[color]] = perm[i]
-            counts[color] += 1
-        # print ("self.bdrylabels", self.bdrylabels)
-
     def _constructNormalsAndAreas(self):
         elem = self.simplices
         self.sigma = np.array([2 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic)-1 for ic in range(self.ncells)])
@@ -373,34 +285,25 @@ class SimplexMesh(object):
                 xt = np.mean(self.points[self.simplices[i1]], axis=0) - np.mean(self.points[self.simplices[i0]], axis=0)
                 if np.dot(self.normals[i], xt) < 0:  self.normals[i] *= -1
         # self.sigma = np.array([1.0 - 2.0 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic) for ic in range(self.ncells)])
-
+    # ----------------------------------------------------------------#
     def write(self, filename, dirname = None, point_data=None):
-        cell_data_meshio = {}
-        if hasattr(self,'vertex_labels'):
-            cell_data_meshio['vertex'] = {}
-            cell_data_meshio['vertex']['gmsh:physical'] = self.vertex_labels
-        if self.dimension ==2:
-            cells = {'triangle': self.simplices}
-            cells['line'] = self._facedata[0]
-            cell_data_meshio['line']={}
-            cell_data_meshio['line']['gmsh:physical'] = self._facedata[1]
-            cell_data_meshio['triangle']={}
-            cell_data_meshio['triangle']['gmsh:physical'] = self.cell_labels
-        else:
-            cells = {'tetra': self.simplices}
-            cells['triangle'] = self._facedata[0]
-            cell_data_meshio['triangle']={}
-            cell_data_meshio['triangle']['gmsh:physical'] = self._facedata[1]
-            cell_data_meshio['tetra']={}
-            cell_data_meshio['tetra']['gmsh:physical'] = self.cell_labels
         if dirname is not None:
             dirname = dirname + os.sep + "mesh"
             if not os.path.isdir(dirname) :
                 os.makedirs(dirname)
             filename = os.path.join(dirname, filename)
-        print("cell_data_meshio['line']['gmsh:physical']", cell_data_meshio['line']['gmsh:physical'])
-        meshio.write_points_cells(filename=filename, points=self.points, cells=cells, point_data=point_data, cell_data=cell_data_meshio, file_format='gmsh2-ascii')
-
+        if self.dimension == 1:
+            cells = {'lines': self.simplices}
+            cells['vertex'] = self.facesdata
+        elif self.dimension ==2:
+            cells = {'triangle': self.simplices}
+            cells['line'] = self.facesdata
+        else:
+            cells = {'tetra': self.simplices}
+            cells['triangle'] = self.facesdata
+        mesh = meshio.Mesh(self.points, cells)
+        meshio.write(filename, mesh)
+    # ----------------------------------------------------------------#
     def computeSimpOfVert(self, test=False):
         S = sparse.dok_matrix((self.nnodes, self.ncells), dtype=int)
         for ic in range(self.ncells):
@@ -417,31 +320,27 @@ class SimplexMesh(object):
             plotmesh.meshWithNodesAndTriangles(meshdata)
             plt.show()
 
-    def plot(self, **kwargs):
-        from simfempy.meshes import plotmesh
-        plotmesh.plotmesh(self, **kwargs)
-    def plotWithBoundaries(self):
-        # from . import plotmesh
-        from simfempy.meshes import plotmesh
-        plotmesh.meshWithBoundaries(self)
-    def plotWithNumbering(self, **kwargs):
-        # from . import plotmesh
-        from simfempy.meshes import plotmesh
-        plotmesh.plotmeshWithNumbering(self, **kwargs)
-    def plotWithData(self, **kwargs):
-        # from . import plotmesh
-        from simfempy.meshes import plotmesh
-        plotmesh.meshWithData(self, **kwargs)
-
-
 #=================================================================#
 if __name__ == '__main__':
-    import geomdefs
-    geometry = geomdefs.unitsquare.Unitsquare()
-    mesh = SimplexMesh(geometry=geometry, hmean=2)
+    import pygmsh
+    rect = [-2, 2, -2, 2]
+    with pygmsh.geo.Geometry() as geom:
+        z=0
+        xc, yc, r = 0.5, 0.5, 0.5
+        mesh_size = 0.1
+        hole = geom.add_circle(x0=[xc,yc], radius=r, mesh_size=mesh_size, num_sections=6, make_surface=False)
+        lines = hole.curve_loop.curves
+        geom.add_physical(lines, label="3000")
+        holes = [hole]
+        p = geom.add_rectangle(*rect, z=0, mesh_size=1, holes=holes)
+        geom.add_physical(p.surface, label="100")
+        for i in range(len(p.lines)): geom.add_physical(p.lines[i], label=f"{1000 + i}")
+        mesh = geom.generate_mesh()
+    print(f"{mesh=}")
+    mesh = SimplexMesh(mesh=mesh)
     import plotmesh
     import matplotlib.pyplot as plt
     fig, axarr = plt.subplots(2, 1, sharex='col')
     plotmesh.meshWithBoundaries(mesh, ax=axarr[0])
     plotmesh.plotmeshWithNumbering(mesh, ax=axarr[1])
-    plotmesh.plotmeshWithNumbering(mesh, localnumbering=True)
+    # plotmesh.plotmeshWithNumbering(mesh, localnumbering=True)
