@@ -22,54 +22,54 @@ class CompareMethods(object):
     def __init__(self, methods, **kwargs):
         self.methods = methods
         self.dirname = "Results"
-        self.latex = True
-        self.plot = False
-        self.plotpostprocs = False
         if 'clean' in kwargs and kwargs.pop("clean")==True:
             import os, shutil
             try: shutil.rmtree(os.getcwd() + os.sep + self.dirname)
             except: pass
-        if 'latex' in kwargs: self.latex = kwargs.pop("latex")
-        if 'plot' in kwargs: self.plot = kwargs.pop("plot")
-        if 'plotpostprocs' in kwargs: self.plotpostprocs = kwargs.pop("plotpostprocs")
-        if 'verbose' in kwargs:
-            verbose = int(kwargs.pop("verbose"))
-            if verbose > 0: self.latex = True
-            if verbose > 1: self.plotpostprocs = True
-            if verbose > 2: self.plot = True
-        if 'h' in kwargs:
-            self.h = kwargs.pop("h")
-            self.paramname = kwargs.pop("paramname")
-        else:
-            self.paramname = "ncells"
-            if 'niter' in kwargs: self.niter = kwargs.pop("niter")
-            else: self.niter = -1
+        self.verbose = kwargs.pop("verbose", 1)
+        self.latex = kwargs.pop("latex", True)
+        self.plotsolution = kwargs.pop("plotsolution", False)
+        self.plotpostprocs = kwargs.pop("plotpostprocs", False)
+        if self.verbose == 0: self.latex = False
+        self.paramname = kwargs.pop("paramname", "ncells")
+        self.plot = kwargs.pop("plot", False)
+        self.createMesh = kwargs.pop("createMesh", None)
+        self.geom = kwargs.pop("geom", None)
+        self.mesh = kwargs.pop("mesh", None)
+        self.h = kwargs.pop("h", None)
         self.parameters = []
-    def _mesh_from_geom_or_fct(self, h):
-        if hasattr(self,"createMesh"): return self.createMesh(h)
+    def _mesh_from_geom_or_fct(self, h=None):
+        if h is None:
+            if self.mesh is not None: return self.mesh
+            if self.h is None: raise ValueError(f"I need h({self.h=})")
+            h = self.h
+        if self.createMesh is not None: return self.createMesh(h)
         if hasattr(pygmsh, "built_in"):
             mesh = pygmsh.generate_mesh(self.geom(h), verbose=False)
         else:
             with self.geom(h) as geom:
                 mesh = geom.generate_mesh()
         return simfempy.meshes.simplexmesh.SimplexMesh(mesh=mesh)
-
-    def compare(self, createMesh=None, geom=None, h=None, params=None):
-        if createMesh is None: self.geom = geom
-        else: self.createMesh = createMesh
+    def compare(self, h=None, params=None, niter=None):
         if self.paramname == "ncells":
             if h is None:
-                mesh = self._mesh_from_geom_or_fct(self.h)
+                mesh = self._mesh_from_geom_or_fct()
                 gmshrefine = True
-                if self.niter ==-1: raise KeyError("please give 'niter'")
-                params = [mesh.ncells*mesh.dimension**i for i in range(self.niter)]
+                if niter is None: raise KeyError("please give 'niter' ({self.paramname=}")
+                params = [mesh.ncells*mesh.dimension**i for i in range(niter)]
             else:
                 params = h
                 gmshrefine = False
         else:
-            mesh = self._mesh_from_geom_or_fct(self.h)
+            mesh = self._mesh_from_geom_or_fct()
             # mesh = SimplexMesh(geometry=geometry, hmean=self.h)
+        if self.plotsolution:
+            import matplotlib.gridspec as gridspec
+            fig = plt.figure(figsize=(10, 8))
+            outer = gridspec.GridSpec(1, len(params)*len(self.methods), wspace=0.2, hspace=0.2)
+            plotcount = 0
         for iter, param in enumerate(params):
+            if self.verbose: print(f"{self.paramname=} {param=}")
             if self.paramname == "ncells":
                 if gmshrefine:
                     mesh = simfempy.meshes.pygmshext.gmshRefine(mesh)
@@ -79,19 +79,21 @@ class CompareMethods(object):
             else:
                 self.parameters.append(param)
             for name, method in self.methods.items():
+                if self.verbose: print(f"{method:-}")
                 method.setMesh(mesh)
                 self.dim = mesh.dimension
                 if self.paramname != "ncells": method.setParameter(self.paramname, param)
-                result = method.solve(iter, self.dirname)
-                if self.plot:
-                    from ..meshes import plotmesh
+                result = method.solve(self.dirname)
+                if self.plotsolution:
+                    from simfempy.meshes import plotmesh
                     suptitle = "{}={}".format(self.paramname, self.parameters[-1])
-                    plotmesh.meshWithData(mesh, data=result.data, title=name, suptitle=suptitle)
-                    plt.show()
+                    plotmesh.meshWithData(mesh, data=result.data, title=name, suptitle=suptitle, fig=fig, outer=outer[plotcount])
+                    plotcount += 1
+                    # plt.show()
                 resdict = result.info.copy()
                 resdict.update(result.data['global'])
-                # print(resdict)
                 self.fillInfo(iter, name, resdict, len(params))
+        if self.plotsolution: plt.show()
         if self.plotpostprocs:
             self.plotPostprocs(self.methods.keys(), self.paramname, self.parameters, self.infos)
         if self.latex:
@@ -103,13 +105,29 @@ class CompareMethods(object):
             self.infos = {}
             for key2, info2 in info.items():
                 self.infos[key2] = {}
-                for key3, info3 in info2.items():
-                    self.infos[key2][key3] = {}
+                if isinstance(info2, dict):
+                    for key3, info3 in info2.items():
+                        self.infos[key2][key3] = {}
+                        for name2 in self.methods.keys():
+                            self.infos[key2][key3][name2] = np.zeros(shape=(n), dtype=type(info3))
+                elif isinstance(info2, simfempy.tools.timer.Timer):
+                    for key3, info3 in info2.data.items():
+                        self.infos[key2][key3] = {}
+                        for name2 in self.methods.keys():
+                            self.infos[key2][key3][name2] = np.zeros(shape=(n), dtype=type(info3))
+                else:
                     for name2 in self.methods.keys():
-                        self.infos[key2][key3][name2] = np.zeros(shape=(n), dtype=type(info3))
+                        self.infos[key2][name2] = np.zeros(shape=(n), dtype=type(info2))
         for key2, info2 in info.items():
-            for key3, info3 in info2.items():
-                self.infos[key2][key3][name][iter] = np.sum(info3)
+            if isinstance(info2, dict):
+                for key3, info3 in info2.items():
+                    self.infos[key2][key3][name][iter] = np.sum(info3)
+            elif isinstance(info2, simfempy.tools.timer.Timer):
+                for key3, info3 in info2.data.items():
+                    self.infos[key2][key3][name][iter] = np.sum(info3)
+            else:
+                self.infos[key2][name][iter] = np.sum(info2)
+
     def generateLatex(self, names, paramname, parameters, infos):
         mesh = self.createMesh.__name__
         title = f"mesh({mesh})\\\\"
@@ -120,6 +138,7 @@ class CompareMethods(object):
         latexwriter = LatexWriter(dirname=self.dirname, title=title, author=self.__class__.__name__)
         for key, val in infos.items():
             kwargs = {'n': parameters, 'nname': paramname}
+            keysplit = key.split('_')
             if key == 'iter':
                 newdict={}
                 for key2, val2 in val.items():
@@ -139,22 +158,13 @@ class CompareMethods(object):
                     kwargs['percentage'] = True
                     latexwriter.append(**kwargs)
             else:
-                kwargs['redrate'] = (key=="error") and (paramname=="ncells")
+                iserr = len(keysplit) >= 2 and keysplit[0] == 'err'
+                kwargs['redrate'] = iserr and (paramname=="ncells")
                 kwargs['diffandredrate'] = not kwargs['redrate'] and (paramname=="ncells")
                 kwargs['dim'] = self.dim
-                if key=="error" and len(self.methods)==1:
-                    newdict={}
-                    for key2, val2 in val.items():
-                        newdict["{}".format(key2)] = val2[name]
-                    kwargs['name'] = '{}-{}'.format(key, name)
-                    kwargs['values'] = newdict
-                    latexwriter.append(**kwargs)
-                else:
-                    for key2, val2 in val.items():
-                        # print(f"{key=} {key2=}")
-                        kwargs['name'] = '{}-{}'.format(key,key2)
-                        kwargs['values'] = val2
-                        latexwriter.append(**kwargs)
+                kwargs['name'] = '{}'.format(key)
+                kwargs['values'] = val
+                latexwriter.append(**kwargs)
         latexwriter.write()
         latexwriter.compile()
     def computeOrder(self, ncells, values, dim):
@@ -187,8 +197,8 @@ class CompareMethods(object):
                         if self.paramname == "ncells":
                             orders, order = self.computeOrder(parameters, val2[name], self.dim)
                             axs[cr, cc].loglog(parameters, orders, '-', label="order {}".format(order))
-                    else:
-                        axs[cr, cc].plot(parameters, val2[name], '-x', label="{}_{}".format(key2, name))
+                    # else:
+                    #     axs[cr, cc].plot(parameters, val2[name], '-x', label="{}_{}".format(key2, name))
                 axs[cr, cc].legend()
                 if key not in singleplots:
                     axs[cr, cc].set_title("{} {}".format(key, key2))
