@@ -2,7 +2,9 @@ import time
 
 #=================================================================#
 class Timer():
-    def __init__(self, name='', verbose=False):
+    def __del__(self):
+        if self.verbose>1: print(self)
+    def __init__(self, name='', verbose=0):
         self.name = name
         self.verbose = verbose
         self.tlast = time.time()
@@ -10,22 +12,18 @@ class Timer():
         self.counter = 0
     def __repr__(self):
         tall = sum(self.data.values())
-        repr = f"\nTimer({self.name:}) total = {tall:8.2e}\n"
+        repr = f"Timer({self.name:}) total = {tall:8.2e}\n"
         for name, t in self.data.items():
             repr += f"\t{name:12s}: {100*t/tall:5.1f}%  ({t:8.2e})\n"
-        return repr
-
-    def items(self): return self.data.items()
+        return repr[:-1]
+    # def items(self): return self.data.items()
     def add(self, name=None):
         if name is None: name = str(self.counter); self.counter += 1
-        t = time.time()
         if name not in self.data: self.data[name] = 0
+        t = time.time()
         self.data[name] += t - self.tlast
         self.tlast = t
-
-    def __del__(self):
-        if self.verbose: print(self)
-
+        if self.verbose==1: print(f"{name=} {t - self.tlast}\n {self=}")
     def reset(self, name):
         self.data[name] = 0
     def reset_all(self):
